/******************************************************************************
;  *       @型号                 : MC51F7424
;  *       @创建日期             : 2023.02.09
;  *       @公司/作者            : SINOMCU.COM-AE
;  *       @晟矽微官网           : http://www.sinomcu.com/
;  *       @版权                 : 晟矽微电 版权所有.
;  *----------------------摘要描述---------------------------------
;  *    MC51F7424 内部温度传感器应用示例.
;  *ADC自动扫描3个通道:AN4(P26),内部LDO 1.5V及内部温度传感器;软件触发,每500ms触发一次;
;  *主程序中将ADC结果计算为电压及温度并输出;LED2翻转周期1S,Debug_Log重定向到UART1.
;  *详见“readme.txt”.
******************************************************************************/

#include "User_Def.h"

#define REV_STR "2023-02-09"
uint8_t code HW_buff[]="\r\n\r\n***** MC51F7424 ADC Tmp *****\r\n*****rev:"REV_STR" "__TIME__; 

#if(defined(EN_PRINT_UART) && EN_PRINT_UART) 
#define  Debug_Log    printf
#else
#define  Debug_Log 
#endif

#define LED_BLINK_HALF_PRE        5    /*uint 100ms*/
/**
  * @brief  main funtion
  * @param  
  * @retval 
  */
void main(void)
{
  uint8_t xdata LED_count;
  uint16_t xdata ADC_result=0,Voltage_mV=0,VDD_mV =0;
  uint16_t xdata Tmp_fab_num =0,Tmp_fab_ADCvalue  =0;
  float xdata Tmp_fab,ADCGainvalue=0,temperature=0;

  /*system init*/
  DISABLE_GIE();             /*Disable Global Interrupt*/
#if( FSYS_CLOCK ==24000000UL)
  Set_FHS_ClkDiv(FHS_DIV_1); /*SYS_CORE_CLK 24MHz*/ 
#endif
  GPIO_Init();
  TIM0_Init(TIM_TICK_1MS);
  UART1_Init(BAUD_RATE);
  ENABLE_GIE();              /*Ensable Global Interrupt*/
  
  Debug_Log(HW_buff);  
  Debug_Log("\r\n--Inf:System Clock %luHz.",FSYS_CLOCK);
#if(VREF_mV==VDD_USER)
  Debug_Log("\r\n--Inf:Vref VDD is %dmV.",VREF_mV);
#else
  Debug_Log("\r\n--Inf:Vref Internal 1.6V %dmV.",VREF_mV);
#endif

  ADC_Init();
  ADCGainvalue =ADC_GainGet(); 
  Start_ADC_Convert();

  /*Tmp_fab_num are and Tmp_fab_ADCvalue factory calibrated values,see datesheet  description*/ 
  Tmp_fab_num       = Get_TmpFabNum();           /*get Sens_Temp1*/ 
  Tmp_fab           = Tmp_fab_num *TMP_FAB_UNIT; /*calc Tmp_offset*/ 
  Tmp_fab_ADCvalue  = Get_TmpFabAdc();           /*get ADC_Temp1*/ 

  /*application loop*/
  Start_100MS_SF_Timer();
  while (1) {
    if(Get_State_100MS_SF_Timer() == SF_TIMER_OV) {
      Start_100MS_SF_Timer();
      LED_count++;
      if( LED_count ==LED_BLINK_HALF_PRE) {    
        LED_count =0;
        LED2_TOGGLE();

        if(g_ADCScanTrigger == SET) {       /*ADC scan convert finish*/ 
          g_ADCScanTrigger = RESET;

          ADC_result = ADC_Result_Buff[0];
          Voltage_mV =(uint16_t)((float)(ADC_result)*(VREF_mV)/FULL_SCALE/ADCGainvalue); 
          Debug_Log("\r\n--Inf:P26(AN4) ADC value is 0x%X,Voltage %dmV.", ADC_result, Voltage_mV);
          ADC_result = ADC_Result_Buff[1];
          Voltage_mV =(uint16_t)((float)(ADC_result)*(VREF_mV)/FULL_SCALE/ADCGainvalue); 
          Debug_Log("\r\n--Inf:Initernal 1.5V ADC value is 0x%X,Voltage %dmV.", ADC_result, Voltage_mV);
            
          ADC_result = ADC_Result_Buff[2];
          /*ADC 12Bit,mV */ 
          /*temperature = (         Vsense            -         VTmpFab          )           /Avg_Slope + Tmp_fab*/
          /*temperature = (tmp_ADCvalue*USER_VDDA     - Tmp_fab_ADCvalue*VDDA_FAB)/ 4095     /Avg_Slope + Tmp_fab*/
          temperature = ((float)ADC_result*VREF_mV/ADCGainvalue - (float)Tmp_fab_ADCvalue*VDDA_FAB)/FULL_SCALE/AVG_SLOPE + Tmp_fab;   /* calc temperature*/
          Debug_Log("\r\n--Inf:Tmp ADC value is 0x%X,Temperature is %2.2f'C. \r\n",ADC_result,temperature);    
          Start_ADC_Convert();         
        }
      }
    }
  }
}

/**************************** end of file *********************************************/
